package info.textgrid.middleware.test;

import static org.junit.Assert.assertTrue;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.TimeZone;
import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.XMLGregorianCalendar;
import org.junit.Test;
import info.textgrid.middleware.OaipmhUtilities;
import info.textgrid.middleware.oaipmh.Resource.Dates.Date;

/**
 *
 */
public class TestOaipmhUtilities {

  /**
   * 
   */
  @Test
  public void testOmitPrefixFromIdentifier() {

    // Check Handle IDs (dhrep).
    String id1 = "hdl:21.11113/0000-000B-C8EF-7";
    String exp1 = "21.11113/0000-000B-C8EF-7";
    String res1 = OaipmhUtilities.omitPrefixFromIdentifier(id1);
    if (!res1.equals(exp1)) {
      System.out.println(res1 + "!=" + exp1);
      assertTrue(false);
    }

    // Check tgrep set IDs (do not filter project: prefix!).
    String id2 = "project:TGPR-2c283391-6ed5-70b4-2e5e-5501c856bca9";
    String exp2 = "project:TGPR-2c283391-6ed5-70b4-2e5e-5501c856bca9";
    String res2 = OaipmhUtilities.omitPrefixFromIdentifier(id2);
    if (!res2.equals(exp2)) {
      System.out.println(res2 + "!=" + exp2);
      assertTrue(false);
    }

    // Check tgrep identifier.
    String id3 = "textgrid:24gv8.0";
    String exp3 = "24gv8.0";
    String res3 = OaipmhUtilities.omitPrefixFromIdentifier(id3);
    if (!res3.equals(exp3)) {
      System.out.println(res3 + "!=" + exp3);
      assertTrue(false);
    }

    // Check unknown prefix.
    String id4 = "urgli:furgli";
    String exp4 = "urgli:furgli";
    String res4 = OaipmhUtilities.omitPrefixFromIdentifier(id4);
    if (!res4.equals(exp4)) {
      System.out.println(res4 + "!=" + exp4);
      assertTrue(false);
    }

    // Check no prefix.
    String id5 = "URGLARGLAUA";
    String exp5 = "URGLARGLAUA";
    String res5 = OaipmhUtilities.omitPrefixFromIdentifier(id5);
    if (!res5.equals(exp5)) {
      System.out.println(res5 + "!=" + exp5);
      assertTrue(false);
    }

    // Check **: prefix.
    String id6 = "**:21.T11991/0000-001C-2AA6-8";
    String exp6 = "**:21.T11991/0000-001C-2AA6-8";
    String res6 = OaipmhUtilities.omitPrefixFromIdentifier(id6);
    if (!res6.equals(exp6)) {
      System.out.println(res6 + "!=" + exp6);
      assertTrue(false);
    }

    // Check ** prefix.
    String id7 = "**21.T11991/0000-001C-2AA6-8";
    String exp7 = "**21.T11991/0000-001C-2AA6-8";
    String res7 = OaipmhUtilities.omitPrefixFromIdentifier(id7);
    if (!res7.equals(exp7)) {
      System.out.println(res7 + "!=" + exp7);
      assertTrue(false);
    }
  }

  /**
   * 
   */
  @Test
  public void testGetSetSpec() {

    String id = "hdl:21.T11991/0000-001B-4C00-E";

    // Set is empty.
    String set = "";
    String exp = "hdl:21.T11991/0000-001B-4C00-E";
    String setSpec = OaipmhUtilities.getSetSpec(set, "hdl:", id);

    if (!setSpec.equals(exp)) {
      System.out.println(setSpec + " != " + exp);
      assertTrue(false);
    }

    // Set is prefix only.
    set = "hdl:";
    exp = "hdl:21.T11991/0000-001B-4C00-E";
    setSpec = OaipmhUtilities.getSetSpec(set, "hdl:", id);

    if (!setSpec.equals(exp)) {
      System.out.println(setSpec + " != " + exp);
      assertTrue(false);
    }

    // Set is NOT empty.
    set = "project:53281-3452-34-5";
    exp = "project:53281-3452-34-5";
    setSpec = OaipmhUtilities.getSetSpec(set, "project:", id);

    if (!setSpec.equals(exp)) {
      System.out.println(setSpec + " != " + exp);
      assertTrue(false);
    }
  }

  /**
   * 
   */
  @Test
  public void testGetTextGridBaseURI() {

    String uriExpected = "textgrid:12345";

    String uri = "textgrid:12345.13";
    String baseURI = OaipmhUtilities.getTextGridBaseURI(uri);
    if (!baseURI.equals(uriExpected)) {
      System.out.println(baseURI + " != " + uriExpected);
      assertTrue(false);
    }

    uri = "textgrid:12345.1";
    baseURI = OaipmhUtilities.getTextGridBaseURI(uri);
    if (!baseURI.equals(uriExpected)) {
      System.out.println(baseURI + " != " + uriExpected);
      assertTrue(false);
    }

    uri = "textgrid:12345";
    baseURI = OaipmhUtilities.getTextGridBaseURI(uri);
    if (!baseURI.equals(uriExpected)) {
      System.out.println(baseURI + " != " + uriExpected);
      assertTrue(false);
    }

    uri = "textgrid:12345.3847";
    baseURI = OaipmhUtilities.getTextGridBaseURI(uri);
    if (!baseURI.equals(uriExpected)) {
      System.out.println(baseURI + " != " + uriExpected);
      assertTrue(false);
    }
  }

  /**
   * 
   */
  @Test
  public void testGetTextGridRevisionURI() {

    String uriExpected = "textgrid:12345.0";
    String uri = "textgrid:12345";
    String revisionURI = OaipmhUtilities.getTextGridRevisionURI(uri);
    if (!revisionURI.equals(uriExpected)) {
      System.out.println(revisionURI + " != " + uriExpected);
      assertTrue(false);
    }

    uriExpected = "textgrid:12345.15";
    uri = "textgrid:12345.15";
    revisionURI = OaipmhUtilities.getTextGridRevisionURI(uri);
    if (!revisionURI.equals(uriExpected)) {
      System.out.println(revisionURI + " != " + uriExpected);
      assertTrue(false);
    }
  }

  /**
   * @throws ParseException
   */
  @Test
  public void testGetUTCDatestampAsString() throws ParseException {

    String tgDateStamp = "2012-02-10T23:45:00.507+01:00";
    String expectedUTCDate = "2012-02-10T22:45:00Z";

    String utcDate = OaipmhUtilities.getUTCDatestampAsString(tgDateStamp);

    if (!utcDate.equals(expectedUTCDate)) {
      assertTrue(utcDate + " != " + expectedUTCDate, false);
    }
  }

  /**
   * @throws ParseException
   * @throws DatatypeConfigurationException
   */
  @Test
  public void testGetUTCDatestampAsGregorian()
      throws ParseException, DatatypeConfigurationException {

    String tgDateStamp = "2012-02-10T23:45:00.507+01:00";
    String expectedUTCDate = "2012-02-10T22:45:00Z";

    XMLGregorianCalendar utcDate = OaipmhUtilities.getUTCDatestampAsGregorian(tgDateStamp);

    System.out.println("millies: " + System.currentTimeMillis());

    GregorianCalendar greg = new GregorianCalendar();
    greg.setTimeInMillis(System.currentTimeMillis());
    java.util.Date d = greg.getTime();
    System.out.println(d.toGMTString());

    System.out.println("tgdatestamp: " + tgDateStamp);
    System.out.println("utcdate: " + utcDate);
    System.out.println("expectedutcdate: " + expectedUTCDate);

    if (!utcDate.toString().equals(expectedUTCDate)) {
      assertTrue(utcDate + " != " + expectedUTCDate, false);
    }
  }

  /**
   * @throws DatatypeConfigurationException
   * @throws ParseException
   */
  @Test
  public void testIsCorrectFromUntilDate() throws DatatypeConfigurationException, ParseException {

    String date = "1970-12-10T18:30:00";
    List<String> result = new ArrayList<String>();
    if (!OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "1970-12-10T18:30";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "1970-12-10T18";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "1970-12-10";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "1970-12";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "1970";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "kakki";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }
    date = "kakki-fakki";
    if (OaipmhUtilities.isCorrectFromUntilDate(date)) {
      result.add(date);
    }

    if (!result.isEmpty()) {
      assertTrue(result.toString() + ": not ISO8691!", false);
    }
  }

  /**
   * @throws DatatypeConfigurationException
   */
  @Test
  public void testGetCurrentUTCDate() throws DatatypeConfigurationException {

    XMLGregorianCalendar currentDate = OaipmhUtilities.getCurrentUTCDate();

    if (!currentDate.isValid()) {
      assertTrue(currentDate + " is not a valid date!", false);
    }
  }

}
